% check.m: Checks solution found by solv.m for the New Keynesian model
%            with markup and technology shocks by making sure that the
%            equilibrium conditions hold after each type of shock. If
%            the solution is accurate, this program will return a 9x4
%            matrix of zeros.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% check equilibrium conditions after an IS shock

  evec = [ siga 0 0 0 ]';

  st = bigw*evec;
  ft = bigu*st;

  rlag = st(1);
  ylag = st(2);
  plag = st(3);
  at = st(4);
  tt = st(5);
  zt = st(6);
  ert = st(7);

  rt = ft(1);
  yt = ft(2);
  pt = ft(3);

  stp = bigpi*st;
  ftp = bigu*stp;

  rlagp = stp(1);
  ylagp = stp(2);
  plagp = stp(3);
  atp = stp(4);
  ttp = stp(5);
  ztp = stp(6);
  ertp = stp(7);

  rtp = ftp(1);
  ytp = ftp(2);
  ptp = ftp(3);

  eqconda = ones(9,1);

  eqconda(1) = yt - ytp + (1/sigma)*(rt-ptp) - (1/sigma)*(1-rhoa)*at;

  eqconda(2) = phi*(pt-beta*ptp) - (theta-1)*(sigma*yt-zt) + tt;

  eqconda(3) = rtp - rhor*rt - rhop*pt - rhoy*yt - ertp;

  eqconda(4) = atp - rhoa*at;

  eqconda(5) = ttp - rhot*tt;

  eqconda(6) = ztp  - rhoz*zt;

  eqconda(7) = rt - rlagp;

  eqconda(8) = yt - ylagp;

  eqconda(9) = pt - plagp;

% check equilibrium conditions after a markup shock

  evec = [ 0 sigt 0 0 ]';

  st = bigw*evec;
  ft = bigu*st;

  rlag = st(1);
  ylag = st(2);
  plag = st(3);
  at = st(4);
  tt = st(5);
  zt = st(6);
  ert = st(7);

  rt = ft(1);
  yt = ft(2);
  pt = ft(3);

  stp = bigpi*st;
  ftp = bigu*stp;

  rlagp = stp(1);
  ylagp = stp(2);
  plagp = stp(3);
  atp = stp(4);
  ttp = stp(5);
  ztp = stp(6);
  ertp = stp(7);

  rtp = ftp(1);
  ytp = ftp(2);
  ptp = ftp(3);

  eqcondt = ones(9,1);

  eqcondt(1) = yt - ytp + (1/sigma)*(rt-ptp) - (1/sigma)*(1-rhoa)*at;

  eqcondt(2) = phi*(pt-beta*ptp) - (theta-1)*(sigma*yt-zt) + tt;

  eqcondt(3) = rtp - rhor*rt - rhop*pt - rhoy*yt - ertp;

  eqcondt(4) = atp - rhoa*at;

  eqcondt(5) = ttp - rhot*tt;

  eqcondt(6) = ztp  - rhoz*zt;

  eqcondt(7) = rt - rlagp;

  eqcondt(8) = yt - ylagp;

  eqcondt(9) = pt - plagp;

% check equilibrium conditions after a technology shock

  evec = [ 0 0 sigz 0 ]';

  st = bigw*evec;
  ft = bigu*st;

  rlag = st(1);
  ylag = st(2);
  plag = st(3);
  at = st(4);
  tt = st(5);
  zt = st(6);
  ert = st(7);

  rt = ft(1);
  yt = ft(2);
  pt = ft(3);

  stp = bigpi*st;
  ftp = bigu*stp;

  rlagp = stp(1);
  ylagp = stp(2);
  plagp = stp(3);
  atp = stp(4);
  ttp = stp(5);
  ztp = stp(6);
  ertp = stp(7);

  rtp = ftp(1);
  ytp = ftp(2);
  ptp = ftp(3);

  eqcondz = ones(9,1);

  eqcondz(1) = yt - ytp + (1/sigma)*(rt-ptp) - (1/sigma)*(1-rhoa)*at;

  eqcondz(2) = phi*(pt-beta*ptp) - (theta-1)*(sigma*yt-zt) + tt;

  eqcondz(3) = rtp - rhor*rt - rhop*pt - rhoy*yt - ertp;

  eqcondz(4) = atp - rhoa*at;

  eqcondz(5) = ttp - rhot*tt;

  eqcondz(6) = ztp  - rhoz*zt;

  eqcondz(7) = rt - rlagp;

  eqcondz(8) = yt - ylagp;

  eqcondz(9) = pt - plagp;

% check equilibrium conditions after a policy shock

  evec = [ 0 0 0 sigr ]';

  st = bigw*evec;
  ft = bigu*st;

  rlag = st(1);
  ylag = st(2);
  plag = st(3);
  at = st(4);
  tt = st(5);
  zt = st(6);
  ert = st(7);

  rt = ft(1);
  yt = ft(2);
  pt = ft(3);

  stp = bigpi*st;
  ftp = bigu*stp;

  rlagp = stp(1);
  ylagp = stp(2);
  plagp = stp(3);
  atp = stp(4);
  ttp = stp(5);
  ztp = stp(6);
  ertp = stp(7);

  rtp = ftp(1);
  ytp = ftp(2);
  ptp = ftp(3);

  eqcondr = ones(9,1);

  eqcondr(1) = yt - ytp + (1/sigma)*(rt-ptp) - (1/sigma)*(1-rhoa)*at;

  eqcondr(2) = phi*(pt-beta*ptp) - (theta-1)*(sigma*yt-zt) + tt;

  eqcondr(3) = rtp - rhor*rt - rhop*pt - rhoy*yt - ertp;

  eqcondr(4) = atp - rhoa*at;

  eqcondr(5) = ttp - rhot*tt;

  eqcondr(6) = ztp  - rhoz*zt;

  eqcondr(7) = rt - rlagp;

  eqcondr(8) = yt - ylagp;

  eqcondr(9) = pt - plagp;

% report results

  [ eqconda eqcondt eqcondz eqcondr ]