% est.m: Maximizes the (minimizes the negative) log likelihood function
%          for the New Keynesian model with markup and technology
%          shocks.
%
%        When maximizing the log likelihood function, the parameters
%          are transformed to satisfy theoretical restrictions.
%          The log likelihood function with transformed parameters
%          is contained in llfn.m.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% load and detrend full sample

%  global yt pt rt

%  load ypr.dat;

%  yt = ypr(:,1);
%  pt = ypr(:,2);
%  rt = ypr(:,3);

%  bigt = length(yt);

%  trend = 1:bigt;

%  yt = yt - 0.004570*trend';

% load and detrend pre-1980 subsample

%  global yt pt rt

%  load ypr.dat;

%  yt = ypr(1:128,1);
%  pt = ypr(1:128,2);
%  rt = ypr(1:128,3);

%  bigt = length(yt);

%  trend = 1:bigt;

%  yt = yt - 0.005353*trend';

% load and detrend post-1980 subsample

  global yt pt rt

  load ypr.dat;

  yt = ypr(129:217,1);
  pt = ypr(129:217,2);
  rt = ypr(129:217,3);

  bigt = length(yt);

  trend = 1:bigt;

  yt = yt - 0.005073*trend';

% set starting values for constrained, full sample estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr siga sigr ]';

% set starting values for unconstrained, full sample estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr rhottr siga sigt sigr ]';

% set starting values for constrained, pre-1980 estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr siga sigr ]';

% set starting values for unconstrained, pre-1980 estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr rhottr siga sigt sigr ]';

% set starting values for constrained, post-1980 estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr siga sigr ]';

% set starting values for unconstrained, post-1980 estimates

  sigma = 1;
  theta = 6;
  phi = 50;

  yss = exp(mean(yt));
  pss = exp(mean(pt));
  rss = exp(mean(rt));

  beta = pss/rss;

  rhor = 0.5471;
  rhoy = 0.0174;
  rhop = 0.6056;

  rhoatr = 0.0564;
  rhottr = 1/300;
  rhoztr = 0.19;

  siga = 0.0163;
  sigt = 0;
  sigz = 0.007;
  sigr = 0.0026;

  bigtheto = [ rhor rhoy rhop rhoatr rhottr siga sigt sigr ]';

% maximize likelihood

  options(1) = 1;
  options(14) = 10000;

  thetstar = fminu('llfn',bigtheto,options);