% estse.m: Maximizes the (minimizes the negative) log likelihood
%            function for the New Keynesian model with markup and
%            technology shocks.
%
%          When maximizing the log likelihood function, the parameters
%            are transformed to satisfy theoretical restrictions.
%            The log likelihood function with transformed parameters
%            is contained in llfn.m.
%
%          When calculating standard errors, the original
%            parameters are used.  The log likelihood function without
%            transformed parameters is contained in llfnse.m.
%
%          The untransformed parameter estimates are reported as
%            the vector tstar.  The standard errors are reported as
%            the vector sevec.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% load and detrend full sample

%  global yt pt rt scalinv

%  load ypr.dat;

%  yt = ypr(:,1);
%  pt = ypr(:,2);
%  rt = ypr(:,3);

%  bigt = length(yt);

%  trend = 1:bigt;

%  yt = yt - 0.004570*trend';

% load and detrend pre-1980 subsample

%  global yt pt rt scalinv

%  load ypr.dat;

%  yt = ypr(1:128,1);
%  pt = ypr(1:128,2);
%  rt = ypr(1:128,3);

%  bigt = length(yt);

%  trend = 1:bigt;

%  yt = yt - 0.005353*trend';

% load and detrend post-1980 subsample

  global yt pt rt scalinv

  load ypr.dat;

  yt = ypr(129:217,1);
  pt = ypr(129:217,2);
  rt = ypr(129:217,3);

  bigt = length(yt);

  trend = 1:bigt;

  yt = yt - 0.005073*trend';

% set starting values for constrained, full sample estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr siga sigr ]';

%  scalvec = [ 1 10 1 1 10 100 ]';

% set starting values for unconstrained, full sample estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr rhottr siga sigt sigr ]';

%  scalvec = [ 1 10 1 1 1 10 10 100 ]';

% set starting values for constrained, pre-1980 estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr siga sigr ]';

%  scalvec = [ 1 10 1 1 10 100 ]';

% set starting values for unconstrained, pre-1980 estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr rhottr siga sigt sigr ]';

%  scalvec = [ 1 10 1 1 1 10 10 100 ]';

% set starting values for constrained, post-1980 estimates

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = 0.80;
%  rhoy = 0.01;
%  rhop = 0.25;

%  rhoatr = 0.09;
%  rhottr = 1/300;
%  rhoztr = 0.19;

%  siga = 1;
%  sigt = 0;
%  sigz = 0.007;
%  sigr = 1;

%  bigtheto = [ rhor rhoy rhop rhoatr siga sigr ]';

%  scalvec = [ 1 10 1 1 10 100 ]';

% set starting values for unconstrained, post-1980 estimates

  sigma = 1;
  theta = 6;
  phi = 50;

  yss = exp(mean(yt));
  pss = exp(mean(pt));
  rss = exp(mean(rt));

  beta = pss/rss;

  rhor = 0.5471;
  rhoy = 0.0174;
  rhop = 0.6056;

  rhoatr = 0.0564;
  rhottr = 1/300;
  rhoztr = 0.19;

  siga = 0.0163;
  sigt = 0;
  sigz = 0.007;
  sigr = 0.0026;

  bigtheto = [ rhor rhoy rhop rhoatr rhottr siga sigt sigr ]';

  scalvec = [ 1 10 1 1 1 10 10 100 ]';

% maximize likelihood

  options(1) = 1;
  options(14) = 10000;

  thetstar = fminu('llfn',bigtheto,options);

% calculate standard errors for constrained estimates

%  thetstar = real(thetstar);

%  sigma = 1;
% theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = thetstar(1);
%  rhoy = thetstar(2);
%  rhop = thetstar(3);

%  rhoa = (100*thetstar(4))/(1+100*abs(thetstar(4)));
%  rhot = (100*rhottr)/(1+100*abs(rhottr));
%  rhoz = (100*rhoztr)/(1+100*abs(rhoztr));

%  siga = abs(thetstar(5));
%  sigt = 0;
%  sigz = 0.007;
%  sigr = abs(thetstar(6));

%  tstar = [ rhor rhoy rhop rhoa siga sigr ]';

%  scalinv = inv(diag(scalvec));

%  tstars = diag(scalvec)*tstar;

%  fstar = llfnse(tstars);

%  eee = 1e-6;

%  epsmat = eee*eye(6);

%  hessvec = zeros(6,1);

%  for i = 1:6

%    hessvec(i) = llfnse(tstars+epsmat(:,i));

%  end

%  hessmatl = zeros(6,6);

%  for i = 1:6

%    for j = 1:i

%      hessmatl(i,j) = (llfnse(tstars+epsmat(:,i)+epsmat(:,j)) ...
%                        -hessvec(i)-hessvec(j)+fstar)/(eee^2);

%    end

%  end

%  hessmatu = hessmatl' - diag(diag(hessmatl));

%  hessmatf = hessmatl + hessmatu;

%  bighx = scalinv*inv(hessmatf)*scalinv';

%  sevec = sqrt(diag(bighx));

% calculate standard errors for unconstrained estimates

  thetstar = real(thetstar);

  sigma = 1;
  theta = 6;
  phi = 50;

  yss = exp(mean(yt));
  pss = exp(mean(pt));
  rss = exp(mean(rt));

  beta = pss/rss;

  rhor = thetstar(1);
  rhoy = thetstar(2);
  rhop = thetstar(3);

  rhoa = (100*thetstar(4))/(1+100*abs(thetstar(4)));
  rhot = (100*thetstar(5))/(1+100*abs(thetstar(5)));
  rhoz = (100*rhoztr)/(1+100*abs(rhoztr));

  siga = abs(thetstar(6));
  sigt = abs(thetstar(7));
  sigz = 0.007;
  sigr = abs(thetstar(8));

  tstar = [ rhor rhoy rhop rhoa rhot siga sigt sigr ]';

  scalinv = inv(diag(scalvec));

  tstars = diag(scalvec)*tstar;

  fstar = llfnse(tstars);

  eee = 1e-6;

  epsmat = eee*eye(8);

  hessvec = zeros(8,1);

  for i = 1:8

    hessvec(i) = llfnse(tstars+epsmat(:,i));

  end

  hessmatl = zeros(8,8);

  for i = 1:8

    for j = 1:i

      hessmatl(i,j) = (llfnse(tstars+epsmat(:,i)+epsmat(:,j)) ...
                        -hessvec(i)-hessvec(j)+fstar)/(eee^2);

    end

  end

  hessmatu = hessmatl' - diag(diag(hessmatl));

  hessmatf = hessmatl + hessmatu;

  bighx = scalinv*inv(hessmatf)*scalinv';

  sevec = sqrt(diag(bighx));