% imp.m: Uses the solution found by solv.m to compute impulse responses
%          for the New Keynesian model with markup and technology
%          shocks.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% IS shock

  sa = zeros(50,7);
  fa = zeros(50,3);

  evec = [ 100*siga 0 0 0 ]';

  sa(2,:) = (bigw*evec)';
  fa(2,:) = (bigu*sa(2,:)')';

  for t = 3:50

    sa(t,:) = (bigpi*sa(t-1,:)')';
    fa(t,:) = (bigu*sa(t,:)')';

  end

% markup shock

  st = zeros(50,7);
  ft = zeros(50,3);

  evec = [ 0 100*sigt 0 0 ]';

  st(2,:) = (bigw*evec)';
  ft(2,:) = (bigu*st(2,:)')';

  for t = 3:50

    st(t,:) = (bigpi*st(t-1,:)')';
    ft(t,:) = (bigu*st(t,:)')';

  end

% technology shock

  sz = zeros(50,7);
  fz = zeros(50,3);

  evec = [ 0 0 100*sigz 0 ]';

  sz(2,:) = (bigw*evec)';
  fz(2,:) = (bigu*sz(2,:)')';

  for t = 3:50

    sz(t,:) = (bigpi*sz(t-1,:)')';
    fz(t,:) = (bigu*sz(t,:)')';

  end

% policy shock

  sr = zeros(50,7);
  fr = zeros(50,3);

  evec = [ 0 0 0 100*sigr ]';

  sr(2,:) = (bigw*evec)';
  fr(2,:) = (bigu*sr(2,:)')';

  for t = 3:50

    sr(t,:) = (bigpi*sr(t-1,:)')';
    fr(t,:) = (bigu*sr(t,:)')';

  end

% create output vectors

  aa = sa(:,4);
  ta = sa(:,5);
  za = sa(:,6);
  epsra = sa(:,7);

  ra = fa(:,1);
  ya = fa(:,2);
  pa = fa(:,3);

  at = st(:,4);
  tt = st(:,5);
  zt = st(:,6);
  epsrt = st(:,7);

  rt = ft(:,1);
  yt = ft(:,2);
  pt = ft(:,3);

  az = sz(:,4);
  tz = sz(:,5);
  zz = sz(:,6);
  epsrz = sz(:,7);

  rz = fz(:,1);
  yz = fz(:,2);
  pz = fz(:,3);

  ar = sr(:,4);
  tr = sr(:,5);
  zr = sr(:,6);
  epsrr = sr(:,7);

  rr = fr(:,1);
  yr = fr(:,2);
  pr = fr(:,3);