function llfnse = llfnse(bigthet);
% Uses the Kalman filter to evaluate the negative log likelihood
%   function for the New Keynesian model with markup and technology
%   shocks. The parameters are untransformed to facilitate calculation
%   of standard errors.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% define variables and parameters

  global yt pt rt scalinv

  bigt = length(yt);

  bigthet = scalinv*bigthet;

% set up parameters for constrained estimation

%  sigma = 1;
%  theta = 6;
%  phi = 50;

%  yss = exp(mean(yt));
%  pss = exp(mean(pt));
%  rss = exp(mean(rt));

%  beta = pss/rss;

%  rhor = bigthet(1);
%  rhoy = bigthet(2);
%  rhop = bigthet(3);

%  rhoa = bigthet(4);
%  rhot = (1/3)/(1+1/3);
%  rhoz = (100*0.19)/(1+100*abs(0.19));

%  siga = bigthet(5);
%  sigt = 0;
%  sigz = 0.007;
%  sigr = bigthet(6);

% set up parameters for unconstrained estimation

  sigma = 1;
  theta = 6;
  phi = 50;

  yss = exp(mean(yt));
  pss = exp(mean(pt));
  rss = exp(mean(rt));

  beta = pss/rss;

  rhor = bigthet(1);
  rhoy = bigthet(2);
  rhop = bigthet(3);

  rhoa = bigthet(4);
  rhot = bigthet(5);
  rhoz = (100*0.19)/(1+100*abs(0.19));

  siga = bigthet(6);
  sigt = bigthet(7);
  sigz = 0.007;
  sigr = bigthet(8);

% form matrices A, B, and C

  biga = zeros(1,1);
  bigb = zeros(1,5);
  bigc = zeros(1,4);

  biga(1,1) = 1;
  bigb(1,1) = rhor;
  bigb(1,2) = rhoy;
  bigb(1,3) = rhop;
  bigc(1,4) = 1;

% form matrices D, F, G, H, and J

  bigd = zeros(5,5);
  bigf = zeros(5,1);
  bigg = zeros(5,5);
  bigh = zeros(5,1);
  bigj = zeros(5,4);

  bigd(1,4) = 1;
  bigd(1,5) = 1/sigma;
  bigg(1,4) = 1;
  bigh(1,1) = 1/sigma;
  bigj(1,1) = -(1/sigma)*(1-rhoa);

  bigd(2,5) = beta*phi;
  bigg(2,4) = -(theta-1)*sigma;
  bigg(2,5) = phi;
  bigj(2,2) = 1;
  bigj(2,3) = theta - 1;

  bigd(3,1) = 1;
  bigh(3,1) = 1;

  bigd(4,2) = 1;
  bigg(4,4) = 1;

  bigd(5,3) = 1;
  bigg(5,5) = 1;

% form matrix P

  bigp = zeros(4,4);

  bigp(1,1) = rhoa;
  bigp(2,2) = rhot;
  bigp(3,3) = rhoz;

% form matrices K and L

  bigkl1 = inv(bigd+bigf*inv(biga)*bigb);
  bigkl2 = bigg + bigh*inv(biga)*bigb;
  bigkl3 = bigj + bigh*inv(biga)*bigc - bigf*inv(biga)*bigc*bigp;

  bigk = bigkl1*bigkl2;

  bigl = bigkl1*bigkl3;

  bigl1 = bigl(1:3,:);
  bigl2 = bigl(4:5,:);

% form matrices M and N

  [kvec,keig] = eig(bigk);

  keig2 = diag(keig);

  [keig3,kord] = sort(abs(keig2));

  kviol = 0;

  if keig3(3) > 1
    kviol = 1;
  end

  if keig3(4) < 1
    kviol = 1;
  end

  keig4 = keig2(kord);

  bign = diag(keig4);

  bign1 = bign(1:3,1:3);
  bign2 = bign(4:5,4:5);

  bigminv = kvec(:,kord);

  bigm = inv(bigminv);

  bigm11 = bigm(1:3,1:3);
  bigm12 = bigm(1:3,4:5);
  bigm21 = bigm(4:5,1:3);
  bigm22 = bigm(4:5,4:5);

% form matrices Q and R

  bigq1 = bigm11*bigl1 + bigm12*bigl2;
  bigq2 = bigm21*bigl1 + bigm22*bigl2;

  vecr = inv(eye(8)-kron(bigp,inv(bign2)))*bigq2(:);

  bigr = reshape(vecr,2,4);

% form matrices S

  bigs1 = -inv(bigm22)*bigm21;

  bigs2 = -inv(bigm22)*inv(bign2)*bigr;

  bigs34 = bigm11 + bigm12*bigs1;

  bigs3 = inv(bigs34)*bign1*bigs34;

  bigs4 = inv(bigs34)*(bigq1+bign1*bigm12*bigs2-bigm12*bigs2*bigp);

  bigs5a = [ eye(3) ; bigs1 ]; 

  bigs5 = inv(biga)*bigb*bigs5a;

  bigs6a = [ zeros(3,4) ; bigs2 ];

  bigs6 = inv(biga)*bigb*bigs6a + inv(biga)*bigc;

% form matrices PI, W, and U

  bigpi = [ bigs3 bigs4 ; zeros(4,3) bigp ];

  bigw = [ zeros(3,4) ; eye(4) ];

  bigu = [ bigs5 bigs6 ; bigs1 bigs2 ];

  bigpi = real(bigpi);

  bigu = real(bigu);

% form matrices AX, BX, CX, VX, and BVBX

  bigax = bigpi;

  bigbx = bigw;

  bigcx = [ bigu(2,:) ; bigu(3,:) ; bigu(1,:) ];

  bigvx = diag([siga^2 sigt^2 sigz^2 sigr^2]);

  bigbvbx = bigbx*bigvx*bigbx';

% put data in deviation form

  ythat = yt - log(yss);

  pthat = pt - log(pss);

  rthat = rt - log(rss);

  dthat = [ ythat pthat rthat ];

% evaluate negative log likelihood

  st = zeros(7,1);

  bigsig1 = inv(eye(49)-kron(bigax,bigax))*bigbvbx(:);

  bigsigt = reshape(bigsig1,7,7);

  llfnse = (3*bigt/2)*log(2*pi);

  for t = 1:bigt

    ut = dthat(t,:)' - bigcx*st;

    omegt = bigcx*bigsigt*bigcx';

    omeginvt = inv(omegt);

    llfnse = llfnse + (1/2)*(log(det(omegt))+ut'*omeginvt*ut);

    bigkt = bigax*bigsigt*bigcx'*omeginvt;

    st = bigax*st + bigkt*ut;

    bigsigt = bigbvbx + bigax*bigsigt*bigax' ...
                - bigax*bigsigt*bigcx'*omeginvt*bigcx*bigsigt*bigax';

  end

% penalize eigenvalue constraint violations

  if kviol

    llfnse = llfnse + 1e12;

  end

  if abs(imag(llfnse)) > 0

    llfnse = real(llfnse) + 1e12;

  end