% solv.m: Solves the New Keynesian model with markup and technology
%           shocks.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% set parameter values

  sigma = 1;
  theta = 6;
  phi = 50;

  yss = 4092.1;
  pss = 1.0087;
  rss = 1.0127;

  beta = pss/rss;

  rhor = 0.7919;
  rhoy = 0.0017;
  rhop = 0.2503;

  rhoa = 0.9590;
  rhot = 0.9672;
  rhoz = 0.95;

  siga = 0.0249;
  sigt = 0.0729;
  sigz = 0.007;
  sigr = 0.0022;

% form matrices A, B, and C

  biga = zeros(1,1);
  bigb = zeros(1,5);
  bigc = zeros(1,4);

  biga(1,1) = 1;
  bigb(1,1) = rhor;
  bigb(1,2) = rhoy;
  bigb(1,3) = rhop;
  bigc(1,4) = 1;

% form matrices D, F, G, H, and J

  bigd = zeros(5,5);
  bigf = zeros(5,1);
  bigg = zeros(5,5);
  bigh = zeros(5,1);
  bigj = zeros(5,4);

  bigd(1,4) = 1;
  bigd(1,5) = 1/sigma;
  bigg(1,4) = 1;
  bigh(1,1) = 1/sigma;
  bigj(1,1) = -(1/sigma)*(1-rhoa);

  bigd(2,5) = beta*phi;
  bigg(2,4) = -(theta-1)*sigma;
  bigg(2,5) = phi;
  bigj(2,2) = 1;
  bigj(2,3) = theta - 1;

  bigd(3,1) = 1;
  bigh(3,1) = 1;

  bigd(4,2) = 1;
  bigg(4,4) = 1;

  bigd(5,3) = 1;
  bigg(5,5) = 1;

% form matrix P

  bigp = zeros(4,4);

  bigp(1,1) = rhoa;
  bigp(2,2) = rhot;
  bigp(3,3) = rhoz;

% form matrices K and L

  bigkl1 = inv(bigd+bigf*inv(biga)*bigb);
  bigkl2 = bigg + bigh*inv(biga)*bigb;
  bigkl3 = bigj + bigh*inv(biga)*bigc - bigf*inv(biga)*bigc*bigp;

  bigk = bigkl1*bigkl2;

  bigl = bigkl1*bigkl3;

  bigl1 = bigl(1:3,:);
  bigl2 = bigl(4:5,:);

% form matrices M and N

  [kvec,keig] = eig(bigk);

  keig2 = diag(keig);

  [keig3,kord] = sort(abs(keig2));

  if keig3(3) > 1
    'error - no solution'
  end

  if keig3(4) < 1
    'error - multiple solutions'
  end

  keig4 = keig2(kord);

  bign = diag(keig4);

  bign1 = bign(1:3,1:3);
  bign2 = bign(4:5,4:5);

  bigminv = kvec(:,kord);

  bigm = inv(bigminv);

  bigm11 = bigm(1:3,1:3);
  bigm12 = bigm(1:3,4:5);
  bigm21 = bigm(4:5,1:3);
  bigm22 = bigm(4:5,4:5);

% form matrices Q and R

  bigq1 = bigm11*bigl1 + bigm12*bigl2;
  bigq2 = bigm21*bigl1 + bigm22*bigl2;

  vecr = inv(eye(8)-kron(bigp,inv(bign2)))*bigq2(:);

  bigr = reshape(vecr,2,4);

% form matrices S

  bigs1 = -inv(bigm22)*bigm21;

  bigs2 = -inv(bigm22)*inv(bign2)*bigr;

  bigs34 = bigm11 + bigm12*bigs1;

  bigs3 = inv(bigs34)*bign1*bigs34;

  bigs4 = inv(bigs34)*(bigq1+bign1*bigm12*bigs2-bigm12*bigs2*bigp);

  bigs5a = [ eye(3) ; bigs1 ]; 

  bigs5 = inv(biga)*bigb*bigs5a;

  bigs6a = [ zeros(3,4) ; bigs2 ];

  bigs6 = inv(biga)*bigb*bigs6a + inv(biga)*bigc;

% form matrices PI, W, and U

  bigpi = [ bigs3 bigs4 ; zeros(4,3) bigp ];

  bigw = [ zeros(3,4) ; eye(4) ];

  bigu = [ bigs5 bigs6 ; bigs1 bigs2 ];

  bigpi = real(bigpi);

  bigu = real(bigu);